#![allow(clippy::too_many_arguments)]

mod game_over;
mod level0;
mod level1;

use crate::game::*;

use bevy::prelude::*;
use std::collections::BTreeSet;

pub fn setup_level(
	commands: &mut Commands,
	current_level: &mut ResMut<CurrentLevel>,
	level_startup_event: &mut EventWriter<LevelStartupEvent>,
	camera_query: &mut Query<&mut Transform, With<Camera>>,
	level_id: LevelId,
) {
	let level_entity = commands
		.spawn()
		.insert(Level)
		.insert(SelectedCharacterId(None))
		.insert(CharacterIdList(BTreeSet::new()))
		.id();
	current_level.0 = Some(level_id);

	camera_query.single_mut().translation = Default::default();

	level_startup_event.send(LevelStartupEvent(level_entity));
}

pub fn despawn_level(mut commands: Commands, level_query: Query<Entity, With<Level>>) {
	for entity in level_query.iter() {
		commands.entity(entity).despawn_recursive();
	}
}

// This is a bad design, to be refactored some days
pub fn post_setup_level(
	mut commands: Commands,
	character_meshes: Res<CharacterMeshes>,
	mut materials: ResMut<Assets<ColorMaterial>>,
	current_level: Res<CurrentLevel>,
	mut level_query: Query<(&mut SelectedCharacterId, &mut CharacterIdList)>,
	mut level_startup_event: EventReader<LevelStartupEvent>,
	asset_server: Res<AssetServer>,
	audio: Res<crossbeam_channel::Sender<AudioMsg>>,
) {
	for LevelStartupEvent(level_entity) in level_startup_event.iter() {
		if let Ok((mut selected_character_id, mut character_id_list)) =
			level_query.get_mut(*level_entity)
		{
			if let Some(level_id) = current_level.0 {
				match level_id.0 {
					0 => level0::setup(
						&mut commands,
						&character_meshes,
						&mut materials,
						&mut selected_character_id,
						&mut character_id_list,
						&audio,
					),
					1 => level1::setup(
						&mut commands,
						&character_meshes,
						&mut materials,
						&mut selected_character_id,
						&mut character_id_list,
						&audio,
					),
					_ => game_over::setup(&mut commands, &asset_server),
				}
			}
		}
	}
}
